﻿using YoutubeExplode.Channels;
using YoutubeExplode.Playlists;
using YoutubeExplode.Videos;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using System.Collections.ObjectModel;

namespace TYTD.Server.Models
{
    public enum InfoType
    {
        Video=0,
        Playlist=1,
        Channel=2,
        User=3,
        ClosedCaptions=4,
        FileDownload=5
    }
    public class MultiDictionary<T1, T2> : IDictionary<T1, T2>
    {
        Dictionary<T1, List<T2>> _dict= new Dictionary<T1, List<T2>>();

        public IReadOnlyDictionary<T1, List<T2>> Dictionary { get { return _dict; } }

        public T2 this[T1 key] { get => throw new System.NotImplementedException(); set => throw new System.NotImplementedException(); }

        public ICollection<T1> Keys => _dict.Keys;

        public ICollection<T2> Values => new Collection<T2>( _dict.Values.Select<List<T2>, T2>(e => e.First()).ToList());

        public int Count => _dict.Count;

        public bool IsReadOnly => false;

        public void Add(T1 key, T2 value)
        {
            if(!_dict.ContainsKey(key))
            {
                _dict.Add(key, new List<T2>());
            }
            _dict[key].Add(value);
        }

        public void Add(KeyValuePair<T1, T2> item)
        {
            Add(item.Key, item.Value);
        }

        public void Clear()
        {
            _dict.Clear();
        }

        public bool Contains(KeyValuePair<T1, T2> item)
        {
            if(_dict.ContainsKey(item.Key))
            {
                return _dict[item.Key].Contains(item.Value);

            }
            return false;
        }

        public bool ContainsKey(T1 key)
        {
            return _dict.ContainsKey(key);
        }

        public void CopyTo(KeyValuePair<T1, T2>[] array, int arrayIndex)
        {
            int i = arrayIndex;
           foreach(var item in _dict)
            {
                var v = new KeyValuePair<T1, T2>(item.Key,item.Value.First());

                array[i++]=v;
            }
        }

        public IEnumerator<KeyValuePair<T1, T2>> GetEnumerator()
        {
            return new TEnumerator(_dict.GetEnumerator());
        }
        private class TEnumerator : IEnumerator<KeyValuePair<T1,T2>>
        {
            public TEnumerator(IEnumerator<KeyValuePair<T1,List<T2>>> dict)
            {
                _enum = dict;
            }
            IEnumerator<KeyValuePair<T1, List<T2>>> _enum;
            public object Current => new KeyValuePair<T1, T2>(_enum.Current.Key, _enum.Current.Value.First());

            KeyValuePair<T1, T2> IEnumerator<KeyValuePair<T1, T2>>.Current => new KeyValuePair<T1, T2>(_enum.Current.Key, _enum.Current.Value.First());

            public void Dispose()
            {
                _enum.Dispose();
            }

            public bool MoveNext()
            {
               return _enum.MoveNext();
            }

            public void Reset()
            {
                _enum.Reset();
            }
        }
        public bool Remove(T1 key)
        {
            return _dict.Remove(key);
        }

        public bool Remove(KeyValuePair<T1, T2> item)
        {
            if(_dict.ContainsKey(item.Key))
            {
                return _dict[item.Key].Remove(item.Value);

            }
            return false;
        }

        public bool TryGetValue(T1 key, out T2 value)
        {
            List<T2> value0;
            if(_dict.TryGetValue(key,out value0))
            {
                value = value0.First();
                return true;
            }
            value = default(T2);
            return false;
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return new TEnumerator(_dict.GetEnumerator());
        }
    }
    public class IDResolutionTypeTriplet
    {
        public string Id { get; set; }
        public InfoType Type { get; set; }
        public Resolution Resolution { get; set; }
        public InfomationQueueItem ToInfomationQueueItem(bool download=true)
        {
            switch(Type)
            {
                case InfoType.Video:
                    VideoId vid = Id;
                   
                        return new InfomationQueueItem(vid, Resolution, download);
                    
                   
                case InfoType.Playlist:
                    PlaylistId pid = Id;
                    return new InfomationQueueItem(pid, Resolution, download);

                  
                case InfoType.Channel:
                    ChannelId cid = Id;
                    return new InfomationQueueItem(cid, Resolution, download);

                case InfoType.User:
                    UserName user = Id;
                   
                        return new InfomationQueueItem(user, Resolution, download);
                case InfoType.ClosedCaptions:
                    VideoId vid2 = Id;
                    return new InfomationQueueItem(vid2);
                   
            }
            return null;
        }
    }
}